<?php
/**
 * User: nester_a
 * Date: 25.06.14
 */

namespace LMS\Libraries\Base;

use LMS\Libraries\Exception\Database;
use LMS\Libraries\Database\Driver;
use LMS\Libraries\ServiceLocator;

class Repository
{
	/**
	 * @var Driver
	 */
	protected $db;

	/**
	 * @var Table
	 */
	protected $table;


	public function __construct($options = array())
	{
		$this->db = isset($options['db']) ? $options['db']  : ServiceLocator::get('db');
	}

	/**
	 * @return Table
	 */
	public function getTable()
	{
		return $this->table;
	}

	/**
	 * @param \LMS\Libraries\Base\Table $table
	 */
	public function setTable($table)
	{
		$this->table = $table;
	}



	/**
	 * @param array $data
	 * @return mixed
	 * @throws Database\Error
	 */
	public function save($data)
	{			
		if(method_exists($this, 'beforeSave')){
			$this->beforeSave($data);
		}

		try{
			$this->table->bind($data);
			$this->table->check();
		}catch(\Exception $e){
			throw new Database\Error($e->getMessage());
		}

		$query = $this->db->getQuery(true);

		if(empty($data[$this->table->getKeyName()])){
			$query->insert($this->db->quoteName($this->table->getTableName()))
					->columns($this->db->quoteName($this->table->getRealColumns()))
					->values(implode(',', $this->table->getValues()));
			$this->db->setQuery($query);

			try{
			    $res = $this->db->execute(); //fix replace query() to execute()
				
				$this->table->{$this->table->getKeyName()} = $this->db->insertId();
				
				return $res;
			}catch(\Exception $e){
				throw new Database\Error($e->getMessage());
			}
		} else {
			return $this->update($data, array($this->table->getKeyName() => (int)$data[$this->table->getKeyName()]));
		}	
	}

	/**
	 * @param array $data
	 * @return mixed
	 * @throws Database\Error
	 */
	public function saveMultiple($data)
	{
		if(method_exists($this, 'beforeSaveMultiple')){
			$this->beforeSaveMultiple($data);
		}

		if(empty($data)){
			return false;
		}

		$columnsAdded = false;

		$query = $this->db->getQuery(true);
		$query->insert($this->db->quoteName($this->table->getTableName()));
		
		foreach($data as $row){
			try{
				$this->table->bind($row);
				$this->table->check();
			}catch(\Exception $e){
				throw new Database\Error($e->getMessage());
			}
			if(!$columnsAdded){
				$query->columns($this->db->quoteName($this->table->getRealColumns()));
				$columnsAdded = true;
			}
			$query->values(implode(',', $this->table->getValues()));
		}
		$this->db->setQuery($query);		

		try{
			return $this->db->execute(); //fix replace query() to execute()
		}catch(\Exception $e){
			throw new Database\Error($e->getMessage());
		}
	}

	/**
	 * @param array $data
	 * @param array $conditions
	 * @return mixed
	 * @throws Database\Error
	 */
	public function update($data, $conditions = array())
	{
		if(method_exists($this, 'beforeUpdate')){
			$this->beforeUpdate($data, $conditions);
		}

		try{
			$this->table->bind($data);
			$this->table->check();
		}catch(\Exception $e){
			throw new Database\Error($e->getMessage());
		}

		$query = $this->db->getQuery(true);
		$fields = array();

		$columns = $this->table->getRealColumns();
		$values = $this->table->getValues();

		foreach($columns as $fieldName){
			if(isset($data[$this->table->getAlias($fieldName)]) || isset($data[$fieldName])){
				$fields[] = $this->db->quoteName($fieldName) . ' = ' . $values[$fieldName];
			}
		}

		$updateConditions = array();
		foreach($conditions as $fieldName => $fieldValue){
			if(is_array($fieldValue)){
				$updateConditions[] = $this->db->quoteName($fieldName) . ' IN (' . implode(',', $fieldValue) . ')';
			}else{
				$updateConditions[] = $this->db->quoteName($fieldName) . ' = ' . $this->db->quote($fieldValue);
			}
		}
			
		$query->update($this->table)
				->set($fields);		
		
		if(!empty($updateConditions)){
			$query->where($updateConditions);
		}

		$this->db->setQuery($query);		
		
		try{				
			return $this->db->query();
		}catch(\Exception $e){
			throw new Database\Error($e->getMessage());
		}
	}

	/**
	 * @param array $conditions
	 * @return mixed
	 * @throws Database\Error
	 */
	public function delete($conditions = array())
	{
		if(!is_array($conditions) && !empty($conditions)) {
			$conditions = array($this->table->getKeyName() => $conditions);
		}
		
		if(method_exists($this, 'beforeDelete')){
			$this->beforeDelete($conditions);
		}

		$query = $this->db->getQuery(true);

		$deleteConditions = array();
		foreach($conditions as $fieldName => $fieldValue){
			if(empty($fieldValue)){
				throw new Database\Error('Wrong delete condition!');
			}
			if(is_array($fieldValue)){
				$deleteConditions[] = $this->db->quoteName($fieldName) . ' IN (' . implode(',', $fieldValue) . ')';
			}else{
				$deleteConditions[] = $this->db->quoteName($fieldName) . ' = ' . $this->db->quote($fieldValue);
			}
		}

		$query->delete($this->table);

		if(!empty($deleteConditions)){
			$query->where($deleteConditions);
		}

		$this->db->setQuery($query);

		try{
			return $this->db->execute(); //fix replace query() to execute()
		}catch(\Exception $e){
			throw new Database\Error($e->getMessage());
		}

	}


	/**
	 * @param array $filterOptions
	 * @return mixed
	 */
	public function getByFilter($filterOptions)
	{
		$query = $this->getByFilterQuery($filterOptions);

		$limit = isset($filterOptions['limit']) ? $filterOptions['limit'] : 0;
		$offset = isset($filterOptions['offset']) ? $filterOptions['offset'] : 0;
		$this->db->setQuery($query, $offset, $limit);
		
		$resultType = isset($filterOptions['resultType'])?$filterOptions['resultType']:'';

		$methodName = 'load'.ucfirst($resultType);
		if(method_exists($this->db, $methodName)){
			return $this->db->$methodName();
		}else{
			return $this->db->loadObjectList();
		}
	}


	/**
	 * @param array $filterOptions
	 * @return \LMS\Libraries\Database\Query
	 */
	public function getByFilterQuery($filterOptions)
	{
		$query = $this->db->getQuery(true);
		$query->from($this->table);

		if(isset($filterOptions['list.ordering'])){
			$query->order($filterOptions['list.ordering'].' '.(isset($filterOptions['list.direction']) ? $filterOptions['list.direction'] : 'ASC' ));
		}

		$query = $this->prepareFilter($query, $filterOptions);

		if(isset($filterOptions['fields'])){
			$query->select($filterOptions['fields']);
		} else {
			$query->select($this->table->getTableAlias().'.*'); //If the model state Fields will not be specified will return an empty result
		}

		return $query;
	}
	
	public function getTotal($filterOptions = array()) 
	{			
		$alias = $this->table->getTableAlias();			
		$query = $this->db->getQuery(true);	
		
		$query->from($this->table);
		$query = $this->prepareFilter($query, $filterOptions);
		$query->select('COUNT('.$alias.'.'.$this->table->getField('id').')');
		
		$this->db->setQuery($query);		
		return $this->db->loadResult();
	}

	/**
	 * Return object by its ID
	 *
	 * @return \stdClass
	 */
	public function getByID($id)
	{
		$query = $this->db->getQuery(true);
		$query->from($this->table);

		$alias = $this->table->getTableAlias();
		$query->select($alias . '.*');
		$query->where($alias . '.' . $this->table->getKeyName() .' = '.$this->db->quote($id));
		$this->db->setQuery($query);
		return $this->db->LoadObject();
	}
	
	/**
	 * @param array $fields	 
	 * @param array $options
	 * @return mixed
	 */
	public function select($fields, $options = array())
	{
		$query = $this->db->getQuery(true);
		
		$query->from($this->table);
		$query->select($fields);
		
		$query = $this->prepareFilter($query, $options);
		
		if(!empty($options['where'])){
			$query->where($options['where']);
		}

		if(!empty($options['order'])){
			$query->order($options['order']);
		}

		if(!empty($options['group'])){
			$query->group($options['group']);
		}

		$limit = isset($options['limit']) ? $options['limit'] : 0;
		$offset = isset($options['offset']) ? $options['offset'] : 0;
				
		$this->db->setQuery($query, $offset, $limit);
		
		$resultType = isset($options['resultType'])?$options['resultType']:'';

		$methodName = 'load'.ucfirst($resultType);
		if(method_exists($this->db, $methodName)){
			return $this->db->$methodName();
		}else{
			return $this->db->loadObjectList();
		}
	}


	/**
	 * @param array $fields
	 * @param array $where
	 * @param array $options
	 * @return mixed
	 */
	public function find($fields, $where=array(), $options = array())
	{
		$query = $this->db->getQuery(true);

		$query->from($this->table);
		$query->select($fields);
		if(!empty($where)){
			$query->where($where);
		}

		if(isset($options['order'])){
			$query->order($options['order']);
		}

		if(isset($options['group'])){
			$query->group($options['group']);
		}


		$limit = isset($options['limit']) ? $options['limit'] : 0;
		$offset = isset($options['offset']) ? $options['offset'] : 0;


		$this->db->setQuery($query, $offset, $limit);
		return $this->db->loadObjectList();
	}


	/**
	 * @return int
	 */
	public function getId(){				
		return $this->table->{$this->table->getKeyName()};
	}

    public function lastInsertedID(){
        return $this->db->insertid();
    }

	/**
	 * @param \JDatabaseQuery $query
	 * @param $filterOptions
	 * @return mixed
	 */
	protected function prepareFilter($query, &$filterOptions)
	{
		if(isset($filterOptions['where'])){
			$query->where($filterOptions['where']);
		}
		
		return $query;
	}
	
	public function getQuery(){
        return (string)$this->db->getQuery();
    }

}